/*
 * Created on: Aug 7, 2024
 */

#include <stdint.h>

#include "hsv2rgb.h"
#include "led.h"
#include "rand.h"



static uint16_t rnd;
static uint16_t work[4];



/*
 *
 */
static void prog_0_flames(uint8_t tick)
{
    uint8_t i;
    uint16_t j;

    uint16_t hue = 0;   // straight red

    if ((tick & 0x3) == 0) {
        for (i = 0; i < 5; i++) {
            work[0] = 8;
            j = prng_get8();
            if (j > 128) {
                work[0] += j >> 1;
            }

            if (j > 64) {
                // put some orange-green hue in there sometimes
                j = (j >= 0xfd) ? (5*6) : 0;

                hsv2rgb_8b(hue + j, 255, work[0] & 0xff,
                        &led.ind.rgb[i][0], &led.ind.rgb[i][1], &led.ind.rgb[i][2]);
            }
        }

        led_matrix_is_updated();
    }

    // update orange more slowly (why? dunno)
    if ((tick & 0x7) == 3) {
        for (i = 0; i < 3; i++) {
            work[0] = 32;
            j = prng_get8();
            if (j > 0x7f) {
                work[0] += j >> 1;
            }

            led.ind.led[i] = work[0];
        }

        led_matrix_is_updated();
    }
}

/*
 * flames
 */
static void prog_1_rainbow(uint8_t tick)
{
    uint8_t i;
    uint16_t hue;

    work[0] += 1;
    work[0] &= 0xff;
    hue = work[0] * 6;

    if (tick & 1) {
        for (i = 0; i < 5; i++) {
            hsv2rgb_8b(hue, 255, 255, &led.ind.rgb[i][0], &led.ind.rgb[i][1], &led.ind.rgb[i][2]);

            hue += 32;
            hue %= 1536;
        }

        for (i = 0; i < 3; i++) {
            led.ind.led[i] = 0;
        }

        led_matrix_is_updated();
    }
}

/*
 * iterate over previous programs after random delays
 */
static void prog_2_iterate(uint8_t tick)
{

}

static void prog_3_off(uint8_t tick)
{
    uint8_t i;

    // blank it
    for (i = 0; i < sizeof(led.all); i++) {
        led.all[i] = 0;
    }

    if (!work[0]) {
        led_matrix_is_updated();
        work[0] = 1;
    }
}



void (*ledprog_flame[4])(uint8_t) = {
    prog_0_flames,
    prog_1_rainbow,
    prog_2_iterate,
    prog_3_off
};



void ledprog_top_init()
{
    uint8_t i;

    rnd = prng_get16();

    // global program initialization
    for (i = 0; i < 4; i++) {
        work[i] = 0;
    }
}
